@@ -158,6 +158,7 @@ try:
 except ImportError:
     from ._cython_compat import FAKE_CYTHON as cython
 
+
 #
 # Alignment padding is handled with the following formula below
 #
@@ -202,8 +203,10 @@ class Unmarshaller:
         "_uint16_unpack",
         "_is_native",
         "_stream_reader",
+        "_sock_reader",
         "_negotiate_unix_fd",
         "_read_complete",
+        "_endian",
     )
 
     def __init__(
@@ -236,6 +239,11 @@ class Unmarshaller:
             if isinstance(stream, io.BufferedRWPair) and hasattr(stream, "reader"):
                 self._stream_reader = stream.reader.read  # type: ignore[attr-defined]
             self._stream_reader = stream.read
+        elif self._negotiate_unix_fd:
+            self._sock_reader = self._sock.recvmsg
+        else:
+            self._sock_reader = self._sock.recv
+        self._endian = 0
 
     def _next_message(self) -> None:
         """Reset the unmarshaller to its initial state.
@@ -280,7 +288,7 @@ class Unmarshaller:
         # This will raise BlockingIOError if there is no data to read
         # which we store in the MARSHALL_STREAM_END_ERROR object
         try:
-            recv = self._sock.recvmsg(missing_bytes, UNIX_FDS_CMSG_LENGTH)  # type: ignore[union-attr]
+            recv = self._sock_reader(missing_bytes, UNIX_FDS_CMSG_LENGTH)  # type: ignore[union-attr]
         except OSError as e:
             errno = e.errno
             if errno == EAGAIN or errno == EWOULDBLOCK:
@@ -311,7 +319,7 @@ class Unmarshaller:
         # which we store in the MARSHALL_STREAM_END_ERROR object
         while True:
             try:
-                data = self._sock.recv(DEFAULT_BUFFER_SIZE)  # type: ignore[union-attr]
+                data = self._sock_reader(DEFAULT_BUFFER_SIZE)  # type: ignore[union-attr]
             except OSError as e:
                 errno = e.errno
                 if errno == EAGAIN or errno == EWOULDBLOCK:
@@ -650,7 +658,9 @@ class Unmarshaller:
         self._msg_len = (
             self._header_len + (-self._header_len & 7) + self._body_len
         )  # align 8
-        self._readers = self._readers_by_type[endian]
+        if self._endian != endian:
+            self._readers = self._readers_by_type[endian]
+            self._endian = endian
 
     def _read_body(self) -> None:
         """Read the body of the message."""
